from math import floor

from features.moving_feature import moving_feature
from features.standard_deviation import standard_deviation
from features.arithmetic_mean import arithmetic_mean
from korpus import create_bewegung, create_bewegung_two_person

CLASS_JOGGEN = 1
CLASS_KNIEBEUGE = -1
CLASS_GEHEN = 0

TRAINING_DATA_PERCENTAGE = 0.9

DATA_LIMIT = 1000
DATA_LIMIT_PER_TYPE = floor(DATA_LIMIT / 2)


def binary_classification_feature(window_size=30):
    # Wir holen uns den Höhensensor des Fußes (Bleibt bei Kniebeugen gleich und bewegt sich beim Laufen)
    joggen_values = create_bewegung('joggen').messungen[1].sensoren[1].werte
    kniebeuge_values = create_bewegung('kniebeuge').messungen[1].sensoren[1].werte
    gehen_values = create_bewegung('gehen').messungen[1].sensoren[1].werte

    # Berechne die Standardabweichung (Bei Kniebeugen gering, bei Joggen hoch)
    joggen_feature = moving_feature(arithmetic_mean, window_size, joggen_values)
    kniebeuge_feature = moving_feature(arithmetic_mean, window_size, kniebeuge_values)
    gehen_feature = moving_feature(arithmetic_mean, window_size, gehen_values)

    return joggen_feature, kniebeuge_feature, gehen_feature

def binary_classification_feature_two_people(window_size=30):
    messung = 0
    sensor = 0
    joggen_values = create_bewegung_two_person('joggen', person=1).messungen[messung].sensoren[sensor].werte
    kniebeuge_values = create_bewegung_two_person('gehen', person=1).messungen[messung].sensoren[sensor].werte
    joggen_values2 = create_bewegung_two_person('joggen', person=2).messungen[messung].sensoren[sensor].werte
    kniebeuge_values2 = create_bewegung_two_person('gehen', person=2).messungen[messung].sensoren[sensor].werte

    # Berechne die Standardabweichung (Bei Kniebeugen gering, bei Joggen hoch)
    joggen_feature = moving_feature(standard_deviation, window_size, list(joggen_values)+list(joggen_values2))
    kniebeuge_feature = moving_feature(standard_deviation, window_size, list(kniebeuge_values) + list(kniebeuge_values2))

    return joggen_feature, kniebeuge_feature


def get_labeled_testdata():
    # Hole die aus den Sensordaten berechneten Merkmale
    joggen_feature, kniebeuge_feature, gehen_feature = binary_classification_feature(200)

    # Wir nehmen nur DATA_LIMIT an Daten (sonst ist K-Nearest-Neighbors zu langsam)
    joggen_feature = joggen_feature[: min(DATA_LIMIT_PER_TYPE, len(joggen_feature))]
    kniebeuge_feature = kniebeuge_feature[: min(DATA_LIMIT_PER_TYPE, len(kniebeuge_feature))]
    gehen_feature = gehen_feature[: min(DATA_LIMIT_PER_TYPE, len(gehen_feature))]

    # Wandel Liste an Merkmalen in einzelne Merkmalsvektoren um
    joggen_vector = ([element] for element in joggen_feature)
    kniebeugen_vector = ([element] for element in kniebeuge_feature)
    gehen_vector = ([element] for element in gehen_feature)

    # Weise den Trainingsdaten eine Klasse zu
    # 0 = Kniebeuge, 1 = Joggen
    training_data_joggen = list(zip(joggen_vector, [CLASS_JOGGEN] * len(joggen_feature)))
    training_data_kniebeuge = list(zip(kniebeugen_vector, [CLASS_KNIEBEUGE] * len(kniebeuge_feature)))
    training_data_gehen = list(zip(gehen_vector, [CLASS_GEHEN] * len(gehen_feature)))

    # Wir nehmen 90 % der Testdaten zum Trainieren und 10 % zum Testen
    delimiter_joggen = floor(len(joggen_feature) * TRAINING_DATA_PERCENTAGE)
    delimiter_kniebeuge = floor(len(kniebeuge_feature) * TRAINING_DATA_PERCENTAGE)
    delimiter_gehen = floor(len(gehen_feature) * TRAINING_DATA_PERCENTAGE)

    training_data = training_data_joggen[:delimiter_joggen] + training_data_kniebeuge[:delimiter_kniebeuge] + training_data_gehen[:delimiter_gehen]
    test_data = training_data_joggen[delimiter_joggen:] + training_data_kniebeuge[delimiter_kniebeuge:] + training_data_gehen[delimiter_gehen:]

    return test_data, training_data
